@@ -356,6 +356,26 @@ def test_pieces():
         ['            str("""',
          '            foo',
          '            """)'],
+        ['            str(f"""',
+         '            {str(str)}',
+         '            """)'],
+        ['            str(f"""',
+         '            foo',
+         '            {',
+         '                str(',
+         '                    str',
+         '                )',
+         '            }',
+         '            bar',
+         '            {str(str)}',
+         '            baz',
+         '            {',
+         '                str(',
+         '                    str',
+         '                )',
+         '            }',
+         '            spam',
+         '            """)'],
         ['def foo2(',
          '        x=1,',
          '        y=2,', '):'],
@@ -514,7 +534,6 @@ def test_invalid_source():
     filename = str(samples_dir / "not_code.txt")
     source = Source.for_filename(filename)
     assert not source.tree
-    assert not hasattr(source, "pieces")
     assert not hasattr(source, "tokens_by_lineno")
 
 
@@ -554,12 +573,12 @@ def test_example():
     result = bar()
     print(result)
     assert result == """\
-bar at line 26
+bar at line 27
 --------------
-  24 | def bar():
-  25 |     <var>names</var> = {}
-  26 >     <exec>exec("result = foo()", globals(), <var>names</var>)</exec>
-  27 |     return <var>names</var>["result"]
+  25 | def bar():
+  26 |     <var>names</var> = {}
+  27 >     <exec>exec("result = foo()", globals(), <var>names</var>)</exec>
+  28 |     return <var>names</var>["result"]
 names = {} 
 
 <module> at line 1
@@ -577,8 +596,9 @@ foo at line 20
 (...)
   18 | <var>            6</var>
   19 | <var>        ][0]</var>)
-  20 >     result = <exec>print_stack()</exec>
-  21 |     return result
+  20 >     result = <exec>print_stack(</exec>
+  21 |              <exec>)</exec>
+  22 |     return result
 [
             1,
             2,
